Class: Bolt::Config
- Inherits:
-
Object
- Object
- Bolt::Config
- Includes:
- Options
- Defined in:
- lib/bolt/config.rb,
lib/bolt/config/options.rb,
lib/bolt/config/modulepath.rb,
lib/bolt/config/transport/lxd.rb,
lib/bolt/config/transport/ssh.rb,
lib/bolt/config/transport/base.rb,
lib/bolt/config/transport/jail.rb,
lib/bolt/config/transport/local.rb,
lib/bolt/config/transport/winrm.rb,
lib/bolt/config/transport/choria.rb,
lib/bolt/config/transport/docker.rb,
lib/bolt/config/transport/podman.rb,
lib/bolt/config/transport/remote.rb,
lib/bolt/config/transport/options.rb
Defined Under Namespace
Modules: Options, Transport Classes: Modulepath
Constant Summary collapse
- DEFAULTS_NAME =
'bolt-defaults.yaml'- DEFAULT_DEFAULT_CONCURRENCY =
The default concurrency value that is used when the ulimit is not low (i.e. < 700)
100
Constants included from Options
Options::CLI_OPTIONS, Options::DEFAULTS_OPTIONS, Options::INVENTORY_OPTIONS, Options::OPTIONS, Options::PLUGIN, Options::PROJECT_OPTIONS, Options::PUPPETDB_OPTIONS, Options::TRANSPORT_CONFIG
Instance Attribute Summary collapse
-
#config_files ⇒ Object
readonly
Returns the value of attribute config_files.
-
#data ⇒ Object
readonly
Returns the value of attribute data.
-
#modified_concurrency ⇒ Object
readonly
Returns the value of attribute modified_concurrency.
-
#project ⇒ Object
readonly
Returns the value of attribute project.
-
#transports ⇒ Object
readonly
Returns the value of attribute transports.
Class Method Summary collapse
- .default ⇒ Object
-
.defaults_schema ⇒ Object
Builds the schema for bolt-defaults.yaml used by the validator.
- .from_project(project, overrides = {}) ⇒ Object
-
.load_bolt_defaults_yaml(dir) ⇒ Object
Loads a ‘bolt-defaults.yaml’ file, which contains default configuration that applies to all projects.
- .load_defaults ⇒ Object
- .system_path ⇒ Object
-
.transport_definitions ⇒ Object
Builds a hash of definitions for transport configuration.
- .user_path ⇒ Object
Instance Method Summary collapse
- #analytics ⇒ Object
- #apply_settings ⇒ Object
-
#check_path_case(type, paths) ⇒ Object
Check if there is a case-insensitive match to the path.
- #color ⇒ Object
- #compile_concurrency ⇒ Object
- #concurrency ⇒ Object
- #deep_clone ⇒ Object
- #default_concurrency ⇒ Object
- #default_inventoryfile ⇒ Object
- #default_puppetdb ⇒ Object
- #disable_warnings ⇒ Object
- #format ⇒ Object
- #format=(value) ⇒ Object
- #future ⇒ Object
- #hiera_config ⇒ Object
-
#initialize(project, config_data, overrides = {}) ⇒ Config
constructor
A new instance of Config.
- #inventoryfile ⇒ Object
- #log ⇒ Object
- #matching_paths(paths) ⇒ Object
-
#merge_config_layers(*config_data) ⇒ Object
Merge configuration from all sources into a single hash.
- #module_install ⇒ Object
- #modulepath ⇒ Object
- #modulepath=(value) ⇒ Object
-
#normalize_overrides(options) ⇒ Object
Transforms CLI options into a config hash that can be merged with default and loaded config.
- #plugin_cache ⇒ Object
- #plugin_hooks ⇒ Object
- #plugins ⇒ Object
- #policies ⇒ Object
- #puppetdb ⇒ Object
- #puppetdb_instances ⇒ Object
- #puppetfile ⇒ Object
- #rerunfile ⇒ Object
- #save_rerun ⇒ Object
-
#sc_open_max_available? ⇒ Boolean
Etc::SC_OPEN_MAX is meaningless on windows, not defined in PE Jruby and not available on some platforms.
- #spinner ⇒ Object
- #stream ⇒ Object
- #trace ⇒ Object
- #transport ⇒ Object
- #trusted_external ⇒ Object
- #validate ⇒ Object
Constructor Details
#initialize(project, config_data, overrides = {}) ⇒ Config
Returns a new instance of Config.
161 162 163 164 165 166 167 168 169 170 171 172 173 174 175 176 177 178 179 180 181 182 183 184 185 186 187 188 189 190 191 192 193 194 195 196 197 198 199 200 201 202 203 204 205 206 207 208 209 210 211 212 213 214 215 216 217 218 219 220 |
# File 'lib/bolt/config.rb', line 161 def initialize(project, config_data, overrides = {}) unless config_data.is_a?(Array) config_data = [{ filepath: project.project_file, data: config_data }] end @logger = Bolt::Logger.logger(self) @project = project @transports = {} @config_files = [] default_data = { 'analytics' => false, 'apply-settings' => {}, 'color' => true, 'compile-concurrency' => Etc.nprocessors, 'concurrency' => default_concurrency, 'disable-warnings' => [], 'format' => 'human', 'log' => { 'console' => {} }, 'module-install' => {}, 'plugin-hooks' => {}, 'plugins' => {}, 'puppetdb' => {}, 'puppetdb-instances' => {}, 'save-rerun' => true, 'spinner' => true, 'transport' => 'ssh' } if project.path.directory? default_data['log']['bolt-debug.log'] = { 'level' => 'debug', 'append' => false } end loaded_data = config_data.each_with_object([]) do |data, acc| if data[:data].any? @config_files.push(data[:filepath]) acc.push(data[:data]) end end override_data = normalize_overrides(overrides) # If we need to lower concurrency and concurrency is not configured ld_concurrency = loaded_data.map(&:keys).flatten.include?('concurrency') @modified_concurrency = default_concurrency != DEFAULT_DEFAULT_CONCURRENCY && !ld_concurrency && !override_data.key?('concurrency') @data = merge_config_layers(default_data, *loaded_data, override_data) TRANSPORT_CONFIG.each do |transport, config| @transports[transport] = config.new(@data.delete(transport), @project.path) end finalize_data validate end |
Instance Attribute Details
#config_files ⇒ Object (readonly)
Returns the value of attribute config_files.
23 24 25 |
# File 'lib/bolt/config.rb', line 23 def config_files @config_files end |
#data ⇒ Object (readonly)
Returns the value of attribute data.
23 24 25 |
# File 'lib/bolt/config.rb', line 23 def data @data end |
#modified_concurrency ⇒ Object (readonly)
Returns the value of attribute modified_concurrency.
23 24 25 |
# File 'lib/bolt/config.rb', line 23 def modified_concurrency @modified_concurrency end |
#project ⇒ Object (readonly)
Returns the value of attribute project.
23 24 25 |
# File 'lib/bolt/config.rb', line 23 def project @project end |
#transports ⇒ Object (readonly)
Returns the value of attribute transports.
23 24 25 |
# File 'lib/bolt/config.rb', line 23 def transports @transports end |
Class Method Details
.default ⇒ Object
30 31 32 |
# File 'lib/bolt/config.rb', line 30 def self.default new(Bolt::Project.default_project, {}) end |
.defaults_schema ⇒ Object
Builds the schema for bolt-defaults.yaml used by the validator.
53 54 55 56 57 58 59 60 61 62 63 |
# File 'lib/bolt/config.rb', line 53 def self.defaults_schema schema = { type: Hash, properties: DEFAULTS_OPTIONS.map { |opt| [opt, _ref: opt] }.to_h, definitions: OPTIONS.merge(transport_definitions) } schema[:definitions]['inventory-config'][:properties] = transport_definitions schema end |
.from_project(project, overrides = {}) ⇒ Object
34 35 36 37 38 39 40 41 |
# File 'lib/bolt/config.rb', line 34 def self.from_project(project, overrides = {}) data = load_defaults.push( filepath: project.project_file, data: project.data ) new(project, data, overrides) end |
.load_bolt_defaults_yaml(dir) ⇒ Object
Loads a ‘bolt-defaults.yaml’ file, which contains default configuration that applies to all projects. This file does not allow project-specific configuration such as ‘hiera-config’ and nests all default inventory configuration under an ‘inventory-config’ key.
82 83 84 85 86 87 88 89 90 91 92 93 94 95 96 97 98 99 100 101 102 103 104 105 106 107 108 109 110 111 112 113 114 115 116 117 118 119 120 121 122 123 124 125 126 127 128 129 130 131 132 133 134 135 136 137 138 139 140 141 142 143 |
# File 'lib/bolt/config.rb', line 82 def self.load_bolt_defaults_yaml(dir) filepath = dir + DEFAULTS_NAME data = Bolt::Util.read_yaml_hash(filepath, 'config') Bolt::Logger.debug("Loaded configuration from #{filepath}") # Validate the config against the schema. This will raise a single error # with all validation errors. Bolt::Validator.new.tap do |validator| validator.validate(data, defaults_schema, filepath) validator.warnings.each { |warning| Bolt::Logger.warn(warning[:id], warning[:msg]) } validator.deprecations.each { |dep| Bolt::Logger.deprecate(dep[:id], dep[:msg]) } end # Remove project-specific config such as hiera-config, etc. project_config = data.slice(*(PROJECT_OPTIONS - DEFAULTS_OPTIONS)) if project_config.any? data.reject! { |key, _| project_config.include?(key) } Bolt::Logger.warn( "unsupported_project_config", "Unsupported project configuration detected in '#{filepath}': #{project_config.keys}. " \ "Project configuration should be set in 'bolt-project.yaml'." ) end # Remove top-level transport config such as transport, ssh, etc. transport_config = data.slice(*INVENTORY_OPTIONS.keys) if transport_config.any? data.reject! { |key, _| transport_config.include?(key) } Bolt::Logger.warn( "unsupported_inventory_config", "Unsupported inventory configuration detected in '#{filepath}': #{transport_config.keys}. " \ "Transport configuration should be set under the 'inventory-config' option or " \ "in 'inventory.yaml'." ) end # Move data under inventory-config to top-level so it can be easily merged with # config from other sources. Error early if inventory-config is not a hash or # has a plugin reference. if data.key?('inventory-config') unless data['inventory-config'].is_a?(Hash) raise Bolt::ValidationError, "Option 'inventory-config' must be of type Hash, received #{data['inventory-config']} " \ "#{data['inventory-config']} (file: #{filepath})" end if data['inventory-config'].key?('_plugin') raise Bolt::ValidationError, "Found unsupported key '_plugin' for option 'inventory-config'; supported keys are " \ "'#{INVENTORY_OPTIONS.keys.join("', '")}' (file: #{filepath})" end data = data.merge(data.delete('inventory-config')) end { filepath: filepath, data: data } end |
.load_defaults ⇒ Object
145 146 147 148 149 150 151 152 153 154 155 156 157 158 159 |
# File 'lib/bolt/config.rb', line 145 def self.load_defaults confs = [] # Load system-level config. if File.exist?(system_path + DEFAULTS_NAME) confs << load_bolt_defaults_yaml(system_path) end # Load user-level config if there is a homedir. if user_path && File.exist?(user_path + DEFAULTS_NAME) confs << load_bolt_defaults_yaml(user_path) end confs end |
.system_path ⇒ Object
65 66 67 68 69 70 71 |
# File 'lib/bolt/config.rb', line 65 def self.system_path if Bolt::Util.windows? Pathname.new(File.join(ENV['ALLUSERSPROFILE'], 'PuppetLabs', 'bolt', 'etc')) else Pathname.new(File.join('/etc', 'puppetlabs', 'bolt')) end end |
.transport_definitions ⇒ Object
Builds a hash of definitions for transport configuration.
45 46 47 48 49 |
# File 'lib/bolt/config.rb', line 45 def self.transport_definitions INVENTORY_OPTIONS.each_with_object({}) do |(option, definition), acc| acc[option] = TRANSPORT_CONFIG.key?(option) ? definition.merge(TRANSPORT_CONFIG[option].schema) : definition end end |
.user_path ⇒ Object
73 74 75 76 77 |
# File 'lib/bolt/config.rb', line 73 def self.user_path Pathname.new(File.(File.join('~', '.puppetlabs', 'etc', 'bolt'))) rescue StandardError nil end |
Instance Method Details
#analytics ⇒ Object
476 477 478 |
# File 'lib/bolt/config.rb', line 476 def analytics @data['analytics'] end |
#apply_settings ⇒ Object
460 461 462 |
# File 'lib/bolt/config.rb', line 460 def apply_settings @data['apply-settings'] end |
#check_path_case(type, paths) ⇒ Object
Check if there is a case-insensitive match to the path
481 482 483 484 485 486 487 488 489 490 491 |
# File 'lib/bolt/config.rb', line 481 def check_path_case(type, paths) return if paths.nil? matches = matching_paths(paths) if matches.any? msg = "WARNING: Bolt is case sensitive when specifying a #{type}. Did you mean:\n" matches.each { |path| msg += " #{path}\n" } Bolt::Logger.warn("path_case", msg) end end |
#color ⇒ Object
420 421 422 |
# File 'lib/bolt/config.rb', line 420 def color @data['color'] end |
#compile_concurrency ⇒ Object
440 441 442 |
# File 'lib/bolt/config.rb', line 440 def compile_concurrency @data['compile-concurrency'] end |
#concurrency ⇒ Object
384 385 386 |
# File 'lib/bolt/config.rb', line 384 def concurrency @data['concurrency'] end |
#deep_clone ⇒ Object
278 279 280 |
# File 'lib/bolt/config.rb', line 278 def deep_clone Bolt::Util.deep_clone(self) end |
#default_concurrency ⇒ Object
509 510 511 512 513 514 515 |
# File 'lib/bolt/config.rb', line 509 def default_concurrency @default_concurrency ||= if !sc_open_max_available? || Etc.sysconf(Etc::SC_OPEN_MAX) >= 300 DEFAULT_DEFAULT_CONCURRENCY else Etc.sysconf(Etc::SC_OPEN_MAX) / 7 end end |
#default_inventoryfile ⇒ Object
356 357 358 |
# File 'lib/bolt/config.rb', line 356 def default_inventoryfile @project.inventory_file end |
#default_puppetdb ⇒ Object
416 417 418 |
# File 'lib/bolt/config.rb', line 416 def default_puppetdb @data['default_puppetdb'] end |
#disable_warnings ⇒ Object
472 473 474 |
# File 'lib/bolt/config.rb', line 472 def disable_warnings Set.new(@project.disable_warnings + @data['disable-warnings']) end |
#format ⇒ Object
388 389 390 |
# File 'lib/bolt/config.rb', line 388 def format @data['format'] end |
#format=(value) ⇒ Object
392 393 394 |
# File 'lib/bolt/config.rb', line 392 def format=(value) @data['format'] = value end |
#future ⇒ Object
396 397 398 |
# File 'lib/bolt/config.rb', line 396 def future @data['future'] end |
#hiera_config ⇒ Object
364 365 366 |
# File 'lib/bolt/config.rb', line 364 def hiera_config @data['hiera-config'] || @project.hiera_config end |
#inventoryfile ⇒ Object
436 437 438 |
# File 'lib/bolt/config.rb', line 436 def inventoryfile @data['inventoryfile'] end |
#log ⇒ Object
404 405 406 |
# File 'lib/bolt/config.rb', line 404 def log @data['log'] end |
#matching_paths(paths) ⇒ Object
493 494 495 |
# File 'lib/bolt/config.rb', line 493 def matching_paths(paths) Array(paths).map { |p| Dir.glob([p, casefold(p)]) }.flatten.uniq.reject { |p| Array(paths).include?(p) } end |
#merge_config_layers(*config_data) ⇒ Object
Merge configuration from all sources into a single hash. Precedence from lowest to highest: defaults, system-wide, user-level, project-level, CLI overrides
252 253 254 255 256 257 258 259 260 261 262 263 264 265 266 267 268 269 270 271 272 273 274 275 276 |
# File 'lib/bolt/config.rb', line 252 def merge_config_layers(*config_data) config_data.inject({}) do |acc, config| acc.merge(config) do |key, val1, val2| case key # Shallow merge config for each plugin when 'plugins' val1.merge(val2) { |_, v1, v2| v1.merge(v2) } # Transports are deep merged when *TRANSPORT_CONFIG.keys Bolt::Util.deep_merge(val1, val2) # Hash values are shallow merged when 'apply-settings', 'log', 'plugin-hooks', 'puppetdb', 'puppetdb-instances' val1.merge(val2) # Disabled warnings are concatenated when 'disable-warnings' val1.concat(val2) when 'analytics' val1 && val2 # All other values are overwritten else val2 end end end end |
#module_install ⇒ Object
468 469 470 |
# File 'lib/bolt/config.rb', line 468 def module_install @project.module_install || @data['module-install'] end |
#modulepath ⇒ Object
372 373 374 |
# File 'lib/bolt/config.rb', line 372 def modulepath (@data['modulepath'] || @project.modulepath) + [@project.managed_moduledir.to_s] end |
#modulepath=(value) ⇒ Object
376 377 378 |
# File 'lib/bolt/config.rb', line 376 def modulepath=(value) @data['modulepath'] = Array(value) end |
#normalize_overrides(options) ⇒ Object
Transforms CLI options into a config hash that can be merged with default and loaded config.
224 225 226 227 228 229 230 231 232 233 234 235 236 237 238 239 240 241 242 243 244 245 246 247 248 |
# File 'lib/bolt/config.rb', line 224 def normalize_overrides() opts = .transform_keys(&:to_s) # Pull out config options. We need to add 'transport' and 'inventoryfile' as they're # not part of the OPTIONS hash but are valid options that can be set with CLI options overrides = opts.slice(*OPTIONS.keys, 'inventoryfile', 'transport', 'default_puppetdb') # Pull out transport config options TRANSPORT_CONFIG.each do |transport, config| overrides[transport] = opts.slice(*config.) end overrides['trace'] = opts['trace'] if opts.key?('trace') # Validate the overrides that can have arbitrary values schema = { type: Hash, properties: CLI_OPTIONS.map { |opt| [opt, _ref: opt] }.to_h, definitions: OPTIONS.merge(INVENTORY_OPTIONS) } Bolt::Validator.new.validate(overrides.slice(*CLI_OPTIONS), schema, 'command line') overrides end |
#plugin_cache ⇒ Object
380 381 382 |
# File 'lib/bolt/config.rb', line 380 def plugin_cache @project.plugin_cache || @data['plugin-cache'] || {} end |
#plugin_hooks ⇒ Object
448 449 450 |
# File 'lib/bolt/config.rb', line 448 def plugin_hooks @data['plugin-hooks'] end |
#plugins ⇒ Object
444 445 446 |
# File 'lib/bolt/config.rb', line 444 def plugins @data['plugins'] end |
#policies ⇒ Object
452 453 454 |
# File 'lib/bolt/config.rb', line 452 def policies @data['policies'] end |
#puppetdb ⇒ Object
408 409 410 |
# File 'lib/bolt/config.rb', line 408 def puppetdb @data['puppetdb'] end |
#puppetdb_instances ⇒ Object
412 413 414 |
# File 'lib/bolt/config.rb', line 412 def puppetdb_instances @data['puppetdb-instances'] end |
#puppetfile ⇒ Object
368 369 370 |
# File 'lib/bolt/config.rb', line 368 def puppetfile @project.puppetfile end |
#rerunfile ⇒ Object
360 361 362 |
# File 'lib/bolt/config.rb', line 360 def rerunfile @project.rerunfile end |
#save_rerun ⇒ Object
424 425 426 |
# File 'lib/bolt/config.rb', line 424 def save_rerun @data['save-rerun'] end |
#sc_open_max_available? ⇒ Boolean
Etc::SC_OPEN_MAX is meaningless on windows, not defined in PE Jruby and not available on some platforms. This method holds the logic to decide whether or not to even consider it.
505 506 507 |
# File 'lib/bolt/config.rb', line 505 def sc_open_max_available? !Bolt::Util.windows? && defined?(Etc::SC_OPEN_MAX) && Etc.sysconf(Etc::SC_OPEN_MAX) end |
#spinner ⇒ Object
428 429 430 |
# File 'lib/bolt/config.rb', line 428 def spinner @data['spinner'] end |
#stream ⇒ Object
432 433 434 |
# File 'lib/bolt/config.rb', line 432 def stream @data['stream'] end |
#trace ⇒ Object
400 401 402 |
# File 'lib/bolt/config.rb', line 400 def trace @data['trace'] end |
#transport ⇒ Object
464 465 466 |
# File 'lib/bolt/config.rb', line 464 def transport @data['transport'] end |
#trusted_external ⇒ Object
456 457 458 |
# File 'lib/bolt/config.rb', line 456 def trusted_external @data['trusted-external-command'] end |
#validate ⇒ Object
335 336 337 338 339 340 341 342 343 344 345 346 347 348 349 350 351 352 353 354 |
# File 'lib/bolt/config.rb', line 335 def validate if @data['modulepath']&.include?(@project.managed_moduledir.to_s) raise Bolt::ValidationError, "Found invalid path in modulepath: #{@project.managed_moduledir}. This path " \ "is automatically appended to the modulepath and cannot be configured." end compile_limit = 2 * Etc.nprocessors unless compile_concurrency < compile_limit raise Bolt::ValidationError, "Compilation is CPU-intensive, set concurrency less than #{compile_limit}" end %w[hiera-config trusted-external-command inventoryfile].each do |opt| Bolt::Util.validate_file(opt, @data[opt]) if @data[opt] end if File.exist?(default_inventoryfile) Bolt::Util.validate_file('inventory file', default_inventoryfile) end end |